# -*- coding: utf-8 -*-
# (c) 2017 Andreas Motl <andreas@hiveeyes.org>
from collections import OrderedDict

def jobee_decode(line):
    """
    Decode JobeeMonitor line format
    See also https://github.com/Jodaille/JobeeMonitor/blob/master/JobeeMonitor/JobeeMonitor.ino

    Example::

        id=Danvou;lux=13940.88;bmpP=997.03;bmpT=20.32;topT=0;entryT=0;h=70.55;siT=19.38;rainLevel=2.24;RainFall=466;milli=332143870;

    """
    data = OrderedDict()
    parts = line.split(';')
    for part in parts:
        try:
            key, value = part.split('=')
            data[key.strip()] = value.strip()
        except:
            pass
    return data


if __name__ == '__main__':
    line = 'id=Danvou;lux=13940.88;bmpP=997.03;bmpT=20.32;topT=0;entryT=0;h=70.55;siT=19.38;rainLevel=2.24;RainFall=466;milli=332143870;'
    print(jobee_decode(line))



# -*- coding: utf-8 -*-
# (c) 2015 Richard Pobering <richard@hiveeyes.org>
# (c) 2015-2018 Andreas Motl <andreas@hiveeyes.org>
import sys
import serial
import logging
from mqtt import BERadioMQTTAdapter
from beradio.decoder import jobee_decode
from beradio.protocol import BencodeError
from beradio.network import protocol_factory
from beradio.util import last_error_and_traceback

"""
Read data in Bencode format from serial port, decode and publish via MQTT.

Based on code from
- Andy Piper http://andypiper.co.uk [2011/09/15]
- Didier Donsez [2014‎/11/22]
  http://air.imag.fr/index.php/Mosquitto#Publication_en_Python

Synopsis::

  python serial_to_mqtt.py /dev/ttyUSB0 localhost
  python serial_to_mqtt.py /dev/ttyACM0 localhost

"""

logger = logging.getLogger(__name__)


class SerialToMQTT(object):

    def __init__(self, serial_device, mqtt_broker, mqtt_topic=None, protocol=2):
        self.serial_device = serial_device
        self.mqtt_broker = mqtt_broker
        self.mqtt_topic = mqtt_topic
        self.protocol_class = protocol_factory(protocol)

    def setup(self):

        try:
            # connect to serial port
            logger.info('Connecting to serial port device "{}"'.format(self.serial_device))
            #self.serial = serial.Serial(serial_device, 9600, timeout=20)
            self.serial = serial.Serial(self.serial_device, 115200)
        except:
            logger.error('Failed to connect to serial port device "{}"'.format(self.serial_device))
            raise

        try:
            logger.info('Connecting to MQTT broker "{}"'.format(self.mqtt_broker))
            self.mqtt = BERadioMQTTAdapter(self.mqtt_broker, topic=self.mqtt_topic, enable_heartbeat=True)
        except:
            logger.error('Failed to connect to MQTT broker "{}"'.format(self.mqtt_broker))
            raise

        return self

    def forward(self):

        try:
            self.serial.flushInput()

            # remain connected to broker
            # read data from serial and publish
            #while self.mqtt.mqttc.loop() == 0:

            # use mqtt.loop_forever() in a separate thread, this will handle
            # automatic reconnecting if connection to broker got lost
            self.mqtt.mqttc.loop_start()

            while True:

                #logger.debug('-' * 42)

                # read line from serial port
                # li999ei99ei1ei2218ei2318ei2462ei2250ee\0\n
                # li999ei99ei1ei2231ei2325ei2443ei2262ee\0\n
                line = self.serial.readline()
                line = line.strip('\r\n\0 ')

                # debug: output line to stdout
                logger.debug('UART data: "{}"'.format(line))

                if line.startswith('#'):
                    continue

                # Decode from CSV format
                if ',' in line:
                    line = line.strip()
                    record = line.split(',')

                    #print 'record:'; pprint(record)

                    # Pop node id from first list element
                    nodeid = record.pop(0)

                    # Ignore empty datetime field
                    datetime = record.pop(0)

                    # The other elements are the data values
                    values = record

                    data = None

                    # Weight,Outside Temp,Outside Humid,Inside Temp,Inside Humid,H1 Temp,H2 Temp,H3 Temp,H4 Temp,H5 Temp,Voltage
                    if nodeid == '2':
                        keys = [
                            'weight',
                            'temp-outside', 'humidity-outside', 'temp-inside', 'humidity-inside',
                            'temp-h1', 'temp-h2', 'temp-h3', 'temp-h4', 'temp-h5', 'voltage'
                        ]
                        data = dict(zip(keys, values))

                    # Weight,Outside Temp,Outside Humid,Inside Temp,Inside Humid,Brood Temp,Voltage
                    elif nodeid == '3':
                        keys = [
                            'weight',
                            'temp-outside', 'humidity-outside', 'temp-inside', 'humidity-inside',
                            'temp-brood', 'voltage',
                        ]
                        data = dict(zip(keys, values))

                    else:
                        logger.error('Could not decode CSV data, unknown nodeid: {}'.format(nodeid))

                    if data:
                        #print 'data:'; pprint(data)

                        for key, value in data.iteritems():
                            data[key] = float(data[key])

                        # Build an appropriate message from CSV data
                        message = {
                            'meta': {
                                'network': self.protocol_class.network_id,
                                'gateway':self.protocol_class.gateway_id,
                                'node': nodeid,
                            },
                            'data': data
                        }
                        self.mqtt.publish_flexible(message)

                    continue


                # Decode JobeeMonitor line format
                # id=Danvou;lux=13940.88;bmpP=997.03;bmpT=20.32;topT=0;entryT=0;h=70.55;siT=19.38;rainLevel=2.24;RainFall=466;milli=332143870;
                if line.startswith('id=') and ';' in line:
                    data = jobee_decode(line)
                    nodeid = data['id']
                    del data['id']

                    # Build an appropriate message from Jobee data
                    message = {
                        'meta': {
                            'network': self.protocol_class.network_id,
                            'gateway':self.protocol_class.gateway_id,
                            'node': nodeid,
                            },
                        'data': data
                    }
                    self.mqtt.publish_flexible(message)

                    continue


                # Decode from Bencode format
                try:
                    data = self.protocol_class.decode(line)
                except BencodeError:
                    continue

                #print 'data:', data

                # publish to MQTT
                if data:
                    raw_sanitized = self.protocol_class.sanitize(line)

                    if self.protocol_class.VERSION == 1:
                        data = self.protocol_class.to_v2(data)

                    self.mqtt.publish_flexible(data, bencode_raw=raw_sanitized)

                #time.sleep(1)

            self.mqtt.mqttc.loop_stop()


        # Handle list index error (i.e. assume no data received)
        except IndexError:
            logger.error('No data received within serial timeout period')
            #self.cleanup()

        # Handle interactive shutdown
        except KeyboardInterrupt:
            logger.info('KeyboardInterrupt received, terminating')
            self.cleanup()

        except Exception as ex:
            logger.error('Unknown exception: {}\n{}'.format(ex, last_error_and_traceback()))


    # called on exit
    # close serial, disconnect MQTT
    def cleanup(self):
        logger.info('Shutting down')
        self.serial.close()
        self.mqtt.close()


if __name__ == '__main__':
    SerialToMQTT(serial_device=sys.argv[1], mqtt_broker=sys.argv[2]).setup().forward()



# -*- coding: utf-8 -*-
# (c) 2015 Richard Pobering <einsiedlerkrebs@netfrag.org>
# (c) 2015 Andreas Motl, Elmyra UG <andreas.motl@elmyra.de>
from pprint import pformat
from beradio.protocol import BERadioProtocol2

class BERadioMessage(object):

    protocol_factory = BERadioProtocol2

    def __init__(self, nodeid, profile='h1'):
        """
        >>> message = BERadioMessage(999)

        >>> str(message)
        'd1:#i999e1:_2:h1e'

        >>> message.temperature(21.63, 19.25, 10.92, 13.54)
        >>> str(message)
        'd1:#i999e1:_2:h11:tli2163ei1925ei1092ei1354eee'

        >>> message.humidity(488.0, 572.0)
        >>> str(message)
        'd1:#i999e1:_2:h11:hli48800ei57200ee1:tli2163ei1925ei1092ei1354eee'

        >>> message.weight(106.77)
        >>> str(message)
        'd1:#i999e1:_2:h11:hli48800ei57200ee1:tli2163ei1925ei1092ei1354ee1:wi10677ee'

        """
        self.nodeid = int(nodeid)
        self.profile = str(profile)
        self.payload = {}
        self.protocol = self.protocol_factory()


    def temperature(self, *args):
        self.payload['t'] = self.protocol.encode_values('t', args)

    def humidity(self, *args):
        self.payload['h'] = self.protocol.encode_values('h', args)

    def weight(self, *args):
        self.payload['w'] = self.protocol.encode_values('w', args)

    def clear(self):
        self.payload = {}

    def encode(self):
        data = {
            '#': self.nodeid,
            '_': self.profile,
        }
        data.update(self.payload)
        message = self.protocol.encode_ether(data)
        return message

    def __str__(self):
        return self.encode()

    def __repr__(self):
        data = {
            '#': self.nodeid,
            '_': self.profile,
            }
        data.update(self.payload)
        return pformat(data)

    @classmethod
    def decode(cls, payload):
        message = cls.protocol_factory().decode(payload)
        message['data'] = dict(message['data'])
        return message



#!/usr/bin/env python
# https://github.com/greghaskins/gibberish
import string
import random

__all__ = ('generate_word', 'generate_words')

initial_consonants = list(set(string.ascii_lowercase) - set('aeiou')
                          # remove those easily confused with others
                          - set('qxc')
# add some crunchy clusters
| set(['bl', 'br', 'cl', 'cr', 'dr', 'fl',
       'fr', 'gl', 'gr', 'pl', 'pr', 'sk',
       'sl', 'sm', 'sn', 'sp', 'st', 'str',
       'sw', 'tr', 'ch', 'sh'])
)

final_consonants = list(set(string.ascii_lowercase) - set('aeiou')
                        # remove the confusables
                        - set('qxcsj')
# crunchy clusters
| set(['ct', 'ft', 'mp', 'nd', 'ng', 'nk', 'nt',
       'pt', 'sk', 'sp', 'ss', 'st', 'ch', 'sh'])
)

vowels = 'aeiou'


def generate_word():
    """Returns a random consonant-vowel-consonant pseudo-word."""
    return ''.join(random.choice(s) for s in (initial_consonants,
                                              vowels,
                                              final_consonants))


def generate_words(wordcount):
    """Returns a list of ``wordcount`` pseudo-words."""
    return [generate_word() for _ in xrange(wordcount)]


def console_main():
    import sys
    try:
        wordcount = int(sys.argv[1])
    except (IndexError, ValueError):
        wordcount = 1
    print(' '.join(generate_words(wordcount)))


if __name__ == '__main__':
    console_main()


# -*- coding: utf-8 -*-
# (c) 2014,2015 Andreas Motl, Elmyra UG <andreas.motl@elmyra.de>
from posix import getpid
from beradio.config import BERadioConfig
from beradio.protocol import BERadioProtocol1, BERadioProtocol2
from beradio.snowflake import SnowflakeGenerator
from beradio.util import PersistentUniqueIdentifier, human_unique_id

class NetworkIdentifier(PersistentUniqueIdentifier):
    store_class = BERadioConfig
    attribute = 'network_id'

class IdentifierSnowflake(PersistentUniqueIdentifier):
    store_class = BERadioConfig
    attribute = 'gateway_id'
    datacenter = 0
    id_generator = SnowflakeGenerator(datacenter, getpid()).get_next_id

class IdentifierHuman(PersistentUniqueIdentifier):
    store_class = BERadioConfig
    attribute = 'gateway_id'
    id_generator = human_unique_id

GatewayIdentifier = IdentifierHuman

def protocol_factory(version=None):

    network_id = str(NetworkIdentifier())
    gateway_id = str(GatewayIdentifier())

    version = str(version)
    if version == '1':
        class_ = BERadioProtocol1
    elif version == '2':
        class_ = BERadioProtocol2
    else:
        class_ = BERadioProtocol2

    return class_(network_id=network_id, gateway_id=gateway_id)



# -*- coding: utf-8 -*-
# (c) 2015 Andreas Motl, Elmyra UG <andreas.motl@elmyra.de>
from beradio import __appname__
from beradio.util import ConfigStoreJson

class BERadioConfig(ConfigStoreJson):
    appname = __appname__



# -*- coding: utf-8 -*-
# (c) 2015 Richard Pobering <richard@hiveeyes.org>
# (c) 2015 Andreas Motl <andreas@hiveeyes.org>
"""BERadio spec and reference implementation"""
__appname__ = u'beradio'
__version__ = u'0.12.0'

def program_name(with_version=False):
    name = __appname__
    if with_version:
        name += u' ' + __version__
    return name



# -*- coding: utf-8 -*-
# (c) 2015 Richard Pobering <einsiedlerkrebs@netfrag.org>
# (c) 2015 Andreas Motl, Elmyra UG <andreas.motl@elmyra.de>
import types
import logging
import bencode
from collections import OrderedDict
from .util import timestamp_nanos

logger = logging.getLogger(__name__)

class BencodeError(Exception):
    pass

class BERadioProtocolBase(object):

    VERSION = None

    def __init__(self, network_id=None, gateway_id=None):
        self.network_id = network_id
        self.gateway_id = gateway_id


    def encode_ether(self, data):
        """
        Encode proper data structure for sending over the air.
        This is just plain Bencode.

        >>> data = {
        ...     '#': 999,
        ...     '_': 'h1',
        ...     't': [2163, 1925, 1092, 1354],
        ...     'h': [488, 572],
        ...     'w': 10677
        ... }

        >>> BERadioProtocolBase().encode_ether(data)
        'd1:#i999e1:_2:h11:hli488ei572ee1:tli2163ei1925ei1092ei1354ee1:wi10677ee'
        """
        payload = bencode.bencode(data)
        return payload


    def decode_ether(self, payload):
        """
        Sanitize and decode from Bencode format.

        >>> BERadioProtocolBase().decode_ether('d1:#i999e1:_2:h11:hli488ei572ee1:tli2163ei1925ei1092ei1354ee1:wi10677ee\\0\\r\\n ')
        {'w': 10677, 'h': [488, 572], '#': 999, 't': [2163, 1925, 1092, 1354], '_': 'h1'}
        """

        # sanitize raw input payload
        payload = self.sanitize(payload)

        # decode from Bencode format
        try:
            data = bencode.bdecode(payload)

        except bencode.BTL.BTFailure as ex:
            raise BencodeError(self.failmsg(ex, payload))

        return data

    @staticmethod
    def sanitize(payload):
        """
        Sanitize raw input payload to make decoding not croak.

        >>> BERadioProtocolBase().sanitize('hello_message\\0\\r\\n ')
        'hello_message'
        """
        return payload.strip('\0\r\n ')


    def failmsg(self, exception, payload):
        msg = 'ERROR: Decoding BERadio version {} data "{}" failed: {}'.format(self.VERSION, payload, exception)
        logger.error(msg)
        return msg

    def decode(self, payload):
        """
        Decoding without implementation should raise an exception.

        >>> be = BERadioProtocolBase()
        >>> be.failmsg = lambda x, y: None
        >>> be.decode('hello_message\\0\\r\\n ')
        Traceback (most recent call last):
          File "beradio/protocol.py", line 89, in decode
            raise NotImplementedError('Please implement in inheriting class.')
        NotImplementedError: Please implement in inheriting class.
        """
        raise NotImplementedError('Please implement in inheriting class.')

    def decode_safe(self, payload):
        try:
            return self.decode(payload)
        except Exception as ex:
            msg = self.failmsg(ex, payload)
            raise


class BERadioProtocol1(BERadioProtocolBase):

    """
    Example payload::

        li999ei99ei1ei2218ei2318ei2462ei2250ee

    Decoded::

        [999, 99, 1, 2218, 2318, 2462, 2250]

    Mapped::

        {
            "network_id": 999,
            "node_id": 99,
            "gateway_id": 1,
            "temp1": 22.18,
            "temp2": 23.18,
            "temp3": 24.62,
            "temp4": 22.5
        }

    """

    VERSION = 1

    # "Bencode-over-Radio" aka. BERadio version 1 field names, order matters.
    # implicitly establishes struct-mapping while decoding raw payloads.
    fieldnames_meta = [
        'network_id', 'node_id', 'gateway_id',
    ]
    fieldnames_data = [
        'temp1', 'temp2', 'temp3', 'temp4',
    ]
    fieldnames = fieldnames_meta + fieldnames_data


    def decode(self, payload):

        data = self.decode_ether(payload)

        # debug: output decoded data to stdout
        logger.info('message v1: {}'.format(data))

        # sanity checks
        # TODO: check exception handling for AssertionError
        assert type(data) is types.ListType, 'Data payload is not a list'

        # decode single values
        #network_id, node_id, gateway_id, temp1, temp2, temp3, temp4 = data
        response = OrderedDict()
        for name, value in zip(self.fieldnames, data):

            # apply inverse scaling
            if name.startswith('temp'):
                value = float(value) / 100

            response[name] = value

        return response


    def to_v2(self, message1):
        message2 = {
            'meta': {
                'network': message1['network_id'],
                'gateway': message1['gateway_id'],
                'node': message1['node_id'],
            },
            'data': { key:value for key, value in message1.items() if key in self.fieldnames_data }
        }
        return message2


class BERadioProtocol2(BERadioProtocolBase):

    """
    Example payload::

        d1:#i999e1:_2:h11:hli488ei572ee1:tli2163ei1925ei1092ei1354ee1:wi10677ee

    Decoded::

        {'w': 10677, 'h': [488, 572], '#': 999, 't': [2163, 1925, 1092, 1354], '_': 'h1'}

    Mapped::

        {
            "meta": {
                "node": "999",
                "profile": "h1",
                "protocol": "beradio2",
                "network": "efc54ed2-b010-42ee-bfb6-183f148885f1",
                "gateway": "tug22"
            },
            "data": {
                "wght1": 106.77,
                "hum1": 488.0,
                "hum2": 572.0,
                "temp1": 21.63,
                "temp2": 19.25,
                "temp3": 10.92,
                "temp4": 13.54
            }
        }

    """

    VERSION = 2

    # BERadio version 2 sensor group identifiers
    # - Expand short names, e.g. "t" to "temp"
    # - Automatically enumerate multiple values and compute appropriate names, e.g. "temp1", "temp2", etc.
    # - Apply proper inverse scaling of sensor values
    identifiers = {
        '#': { 'name': 'node',    'attname' : 'direct', 'meta': True, 'convert': str},
        '_': { 'name': 'profile', 'attname' : 'direct', 'meta': True, 'convert': str},
        't': { 'name': 'temp',    'scale-encode': lambda x: int(x * 100), 'scale-decode': lambda x: float(x) / 100 },
        'h': { 'name': 'hum',     'scale-encode': lambda x: int(x * 100), 'scale-decode': lambda x: float(x) / 100 },
        'w': { 'name': 'wght',    'scale-encode': lambda x: int(x * 100), 'scale-decode': lambda x: float(x) / 100 },
        'r': { 'name': 'rssi',    'scale-encode': lambda x: int(x * 100), 'scale-decode': lambda x: float(x) / 100 },
        'l': { 'name': 'loops',   'scale-encode': lambda x: int(x * 100), 'scale-decode': lambda x: float(x) / 100 },
    }

    def get_envelope(self, node=None):

        # Create nanosecond timestamp
        # TODO: Make this only conditionally?
        timestamp = timestamp_nanos()

        # Prepare response structure
        response = {
            'meta': {
                'protocol': 'beradio2',
                'network': str(self.network_id),
                'gateway': str(self.gateway_id),
                'node': str(node),
                'time': timestamp,
            },
            'data': OrderedDict(),
        }
        return response

    def decode(self, payload):
        """
        Decode BERadio2 message.

        >>> BERadioProtocol2().decode('d1:#i999e1:_2:h11:hli48800ei57200ee1:tli2163ei1925ei1092ei1354ee1:wi10677ee\\0\\r\\n ')  # doctest: +ELLIPSIS
        {'meta': {'node': '999', 'profile': 'h1', 'protocol': 'beradio2', 'network': 'None', 'time': ..., 'gateway': 'None'}, 'data': OrderedDict([('wght1', 106.77), ('hum1', 488.0), ('hum2', 572.0), ('temp1', 21.63), ('temp2', 19.25), ('temp3', 10.92), ('temp4', 13.54)])}
        """

        # Decode data from air
        data_in = self.decode_ether(payload)

        # Debugging: Display decoded data on STDOUT
        logger.debug('message v2: {}'.format(data_in))

        # Sanity checks
        # TODO: check exception handling for AssertionError
        assert type(data_in) is types.DictType, 'Data payload is not a dictionary'

        # Prepare response structure
        response = self.get_envelope()

        # decode nested payload
        for real_identifier, value in data_in.iteritems():

            # Family identifier is the first char
            identifier = real_identifier[0]

            # List continuations: If more digits follow up, decode index offset
            index_offset = 0
            try:
                index_offset = int(real_identifier[1:])
            except:
                pass

            if identifier in self.identifiers:

                rule = self.identifiers.get(identifier)
                name = rule.get('name', identifier)
                is_meta = rule.get('meta', False)

                response_key = 'meta' if is_meta else 'data'

                # multiple values arrive in list
                name_prefix = name
                if type(value) is types.ListType:
                    for idx, item in enumerate(value):
                        index = idx + 1 + index_offset
                        name = name_prefix + str(index)
                        item = self.decode_value(item, rule)
                        response[response_key][name] = item

                # single values arrive as scalar
                else:
                    value = self.decode_value(value, rule)

                    if 'attname' in rule and rule['attname'] == 'direct':
                        pass
                    else:
                        index = 1 + index_offset
                        name += str(index)

                    response[response_key][name] = value

        return response


    def decode_value(self, value, rule):
        if 'convert' in rule:
            value = rule['convert'](value)
        if 'scale-decode' in rule:
            value = rule['scale-decode'](value)
        return value


    def encode_value(self, identifier, value):
        rule = self.identifiers.get(identifier, {})
        if 'scale-encode' in rule:
            value = rule['scale-encode'](value)
        return value

    def encode_values(self, identifier, values):

        # sanity checks
        assert type(values) in (types.ListType, types.TupleType)

        # vararg adaptation
        if type(values) is types.TupleType:
            values = list(values)

        # apply encode scaling for all values
        values = [self.encode_value(identifier, value) for value in values]

        # apply compression on single scalar values
        if len(values) == 1:
            return values[0]

        return values



# -*- coding: utf-8 -*-
# (c) 2014-2018 Andreas Motl <andreas@hiveeyes.org>
import os
import sys
import socket
import logging
import traceback
import json_store
from math import sin
from uuid import uuid4
from appdirs import user_data_dir
from datetime import datetime
from calendar import timegm
from beradio.gibberish import generate_word
from StringIO import StringIO


class Singleton(object):
    """
    Singleton class by Duncan Booth.
    Multiple object variables refers to the same object.
    http://www.suttoncourtenay.org.uk/duncan/accu/pythonpatterns.html
    """
    _instance = None

    def __new__(cls, *args, **kwargs):
        if not cls._instance:
            cls._instance = super(Singleton, cls).__new__(cls)
        return cls._instance


class ConfigStoreJson(dict):

    store = None
    appname = 'DEFAULT'

    def __init__(self):
        self.app_data_dir = user_data_dir(self.appname)
        #print >>sys.stderr, "ConfigStoreJson app_data_dir:", self.app_data_dir
        if not os.path.exists(self.app_data_dir):
            os.makedirs(self.app_data_dir)
        self.config_file = os.path.join(self.app_data_dir, 'config.json')
        #print >>sys.stderr, "ConfigStoreJson config_file:", self.config_file

        self.setup()

    def setup(self):
        if not ConfigStoreJson.store:
            #print "ConfigStoreJson.__init__"
            ConfigStoreJson.store = json_store.open(self.config_file)

    def has_key(self, key):
        return ConfigStoreJson.store.has_key(key)

    def __getitem__(self, key):
        #print 'ConfigStoreJson.__getitem__'
        return ConfigStoreJson.store[key]

    def __setitem__(self, key, value):
        #print 'ConfigStoreJson.__setitem__', key, value
        ConfigStoreJson.store[key] = value
        ConfigStoreJson.store.sync()


class UUIDGenerator(object):

    def get_next_id(self):
        return uuid4()


class PersistentUniqueIdentifier(Singleton):

    config = None
    identifier = 'UNKNOWN'
    attribute = 'uuid'
    store_class = ConfigStoreJson
    id_generator = UUIDGenerator().get_next_id

    def __init__(self):
        if not self.config:
            self.config = self.store_class()
        if not self.config.has_key(self.attribute) or not self.config[self.attribute]:
            self.config[self.attribute] = str(self.id_generator())
        self.identifier = self.config[self.attribute]
        #print >>sys.stderr, self.attribute + ':', self.identifier

    def __str__(self):
        return str(self.identifier)


def human_unique_id(*args):
    """
    Produces random, pronounceable pseudo-words combined with the current second.
    Examples: swan4, zech55, tug22, drew1

    .. seealso::

        https://github.com/greghaskins/gibberish
        http://www.anotherchris.net/csharp/friendly-unique-id-generation-part-2/#time
    """
    now = datetime.now()
    word = generate_word()
    unique = str(word) + str(now.second)
    return unique


def setup_logging(output='-', level=logging.INFO):
    log_format = '%(asctime)-15s [%(name)-20s] %(levelname)-7s: %(message)s'

    if output == '-':
        logging.basicConfig(
            stream=sys.stderr,
            format=log_format,
            level=level)

    else:
        logging.basicConfig(
            filename=output,
            format=log_format,
            level=level)


def timestamp_nanos():
    """
    https://docs.influxdata.com/influxdb/v0.13/troubleshooting/frequently_encountered_issues/#querying-outside-the-min-max-time-range

        Smallest valid timestamp: -9023372036854775808 (approximately 1684-01-22T14:50:02Z)
        Largest valid timestamp: 9023372036854775807 (approximately 2255-12-09T23:13:56Z)
    """
    timestamp = datetime.utcnow()
    # from influxdb.line_protocol._convert_timestamp
    nanos = int(timegm(timestamp.utctimetuple()) * 1e9 + timestamp.microsecond * 1e3)
    return nanos


def math_func(name, x, amplitude=10):
    """
    https://stackoverflow.com/questions/1073606/is-there-a-one-line-function-that-generates-a-triangle-wave

    .. see also::

        - https://docs.scipy.org/doc/scipy-0.14.0/reference/generated/scipy.signal.sawtooth.html
        - https://gist.github.com/endolith/407991
        - https://code.activestate.com/recipes/577592-simple-1khz-audio-function-generator-using-standar/
    """
    x = float(x)
    y = x

    # triangle
    if name == 'triangle':
        y = abs((x % (amplitude * 2)) - amplitude)
        #y = amplitude - abs(x % (2*amplitude) - amplitude)

    # square
    elif name == 'square':
        y = (x % (amplitude * 2)) < amplitude and amplitude or 0

    # sawtooth
    elif name == 'sawtooth':
        y = round((x/amplitude - int(x/amplitude)) * amplitude)

    # sine
    elif name == 'sine':
        y = amplitude * sin(x / 10)

    else:
        raise NotImplementedError('Math func "{}" not implemented'.format(name))

    return y


def traceback_get_exception(num = -1):

    # build error message
    exception_string = ''.join(traceback.format_exception_only(sys.exc_type, hasattr(sys, 'exc_value') and sys.exc_value or 'Unknown'))

    # extract error location from traceback
    if hasattr(sys, 'exc_traceback'):
        (filename, line_number, function_name, text) = traceback.extract_tb(sys.exc_traceback)[num]
    else:
        (filename, line_number, function_name, text) = ('-', '-', '-', '-')

    error = {
        'message': exception_string,
        'location': {
            'filename': filename,
            'line_number': line_number,
            'function_name': function_name,
            'text': text,
            }
    }

    return error


def format_exception_location(error, prefix=''):
    if prefix:
        prefix += "\n"
    error_location =\
    prefix +\
    "Filename:    %s\nLine number: %s\nFunction:    %s\nCode:        %s" %\
    (error['location']['filename'], error['location']['line_number'], error['location']['function_name'], error['location']['text'])
    return error_location


def last_error_and_traceback():
    error_entry = traceback_get_exception(0)
    error_last = traceback_get_exception(-1)
    sep = '-' * 60
    payload_location = "\n".join([sep, format_exception_location(error_entry, "Entry point:"), sep, format_exception_location(error_last, "Source of exception:")])
    payload = ''.join(["ERROR: ", error_entry['message'], "\n", payload_location, "\n"])

    # add full traceback
    buffer = StringIO()
    traceback.print_exc(file=buffer)
    buffer.seek(0)
    payload += '\n' + buffer.read()

    return payload


def get_hostname():
    return socket.gethostname()



# The contents of this file are subject to the BitTorrent Open Source License
# Version 1.1 (the License).  You may not copy or use this file, in either
# source code or executable form, except in compliance with the License.  You
# may obtain a copy of the License at http://www.bittorrent.com/license/.
#
# Software distributed under the License is distributed on an AS IS basis,
# WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License
# for the specific language governing rights and limitations under the
# License.

# Written by Petru Paler

from .BTL import BTFailure


def decode_int(x, f):
    f += 1
    newf = x.index('e', f)
    n = int(x[f:newf])
    if x[f] == '-':
        if x[f + 1] == '0':
            raise ValueError
    elif x[f] == '0' and newf != f+1:
        raise ValueError
    return (n, newf+1)

def decode_string(x, f):
    colon = x.index(':', f)
    n = int(x[f:colon])
    if x[f] == '0' and colon != f+1:
        raise ValueError
    colon += 1
    return (x[colon:colon+n], colon+n)

def decode_list(x, f):
    r, f = [], f+1
    while x[f] != 'e':
        v, f = decode_func[x[f]](x, f)
        r.append(v)
    return (r, f + 1)

def decode_dict(x, f):
    r, f = {}, f+1
    while x[f] != 'e':
        k, f = decode_string(x, f)
        r[k], f = decode_func[x[f]](x, f)
    return (r, f + 1)

decode_func = {}
decode_func['l'] = decode_list
decode_func['d'] = decode_dict
decode_func['i'] = decode_int
decode_func['0'] = decode_string
decode_func['1'] = decode_string
decode_func['2'] = decode_string
decode_func['3'] = decode_string
decode_func['4'] = decode_string
decode_func['5'] = decode_string
decode_func['6'] = decode_string
decode_func['7'] = decode_string
decode_func['8'] = decode_string
decode_func['9'] = decode_string

def bdecode(x):
    try:
        r, l = decode_func[x[0]](x, 0)
    except (IndexError, KeyError, ValueError):
        raise BTFailure("not a valid bencoded string")
    if l != len(x):
        raise BTFailure("invalid bencoded value (data after valid prefix)")
    return r

# MicroPython compatibility
try:
    from types import StringType, IntType, LongType, DictType, ListType, TupleType
except ImportError:
    StringType = str
    IntType = int
    LongType = int
    DictType = dict
    ListType = list
    TupleType = tuple

class Bencached(object):

    __slots__ = ['bencoded']

    def __init__(self, s):
        self.bencoded = s

def encode_bencached(x,r):
    r.append(x.bencoded)

def encode_int(x, r):
    r.extend(('i', str(x), 'e'))

def encode_bool(x, r):
    if x:
        encode_int(1, r)
    else:
        encode_int(0, r)

def encode_string(x, r):
    r.extend((str(len(x)), ':', x))

def encode_list(x, r):
    r.append('l')
    for i in x:
        encode_func[type(i)](i, r)
    r.append('e')

def encode_dict(x,r):
    r.append('d')
    ilist = x.items()
    ilist.sort()
    for k, v in ilist:
        r.extend((str(len(k)), ':', k))
        encode_func[type(v)](v, r)
    r.append('e')

encode_func = {}
encode_func[Bencached] = encode_bencached
encode_func[IntType] = encode_int
encode_func[LongType] = encode_int
encode_func[StringType] = encode_string
encode_func[ListType] = encode_list
encode_func[TupleType] = encode_list
encode_func[DictType] = encode_dict

try:
    from types import BooleanType
    encode_func[BooleanType] = encode_bool
except ImportError:
    pass

def bencode(x):
    r = []
    encode_func[type(x)](x, r)
    return ''.join(r)



# -*- coding: utf-8 -*-
# (c) 2015 Richard Pobering <richard@hiveeyes.org>
# (c) 2015-2018 Andreas Motl <andreas@hiveeyes.org>
import sys
import time
import json
import random
import logging
from beradio.decoder import jobee_decode

try:
    from pprint import pprint
except ImportError:
    from pprint import pformat
    def pprint(x):
        print(pformat(x))
from message import BERadioMessage
from .network import protocol_factory
from .mqtt import BERadioMQTTAdapter
from .util import math_func

"""
Read data in Bencode format from command line, decode and publish via MQTT.

Synopsis::

  # send given Bencode payload
  python publish.py 192.168.59.103 li999ei99ei1ei2218ei2318ei2462ei2250ee

  # send random values
  python publish.py 192.168.59.103 random

"""

logger = logging.getLogger(__name__)

class DataToMQTT(object):

    def __init__(self, mqtt_broker, mqtt_topic=None, protocol=2):
        self.mqtt_broker = mqtt_broker
        self.mqtt_topic = mqtt_topic
        self.protocol_class = protocol_factory(protocol)

    def setup(self):
        try:
            logger.info('Connecting to MQTT broker "{}"'.format(self.mqtt_broker))
            self.mqtt = BERadioMQTTAdapter(self.mqtt_broker, topic=self.mqtt_topic, enable_heartbeat=True)
        except:
            logger.error('Failed to connect to MQTT broker "{}"'.format(self.mqtt_broker))
            raise

        return self

    def publish(self, payload):

        logger.info('Setting up publishing of "{}"'.format(payload))

        # use mqtt.loop_forever() in a separate thread, this will handle
        # automatic reconnecting if connection to broker got lost
        self.mqtt.mqttc.loop_start()

        if payload == 'random':
            if self.protocol_class.VERSION == 1:
                data = [
                    '999', '99', '2',
                    scale_100(random_temp()), scale_100(random_temp()), scale_100(random_temp()), scale_100(random_temp()),
                ]
            elif self.protocol_class.VERSION == 2:
                data = {
                    '#': '999',
                    '_': 'h1',
                    't': [scale_100(random_temp()), scale_100(random_temp()), scale_100(random_temp()), scale_100(random_temp())],
                    'h': [scale_100(random_hum()), scale_100(random_hum())],
                    'w': scale_100(random_weight())}
            payload = self.protocol_class.encode_ether(data)
            logger.info('Publishing random payload: {}'.format(payload))
            self.publish_real(payload)

        elif payload.startswith('func:'):

            func_name = payload.replace('func:', '')

            x = 1.0

            def apply_func(func_name, x, times):
                shifting_factor = 3
                for i in range(times):
                    y = math_func(func_name, x + i * 3)
                    yield y

            #interval = 0
            #interval = 0.01
            interval = 0.5

            logger.info('Running publishing loop with interval={}'.format(interval))
            while True:
                x += 1

                message = BERadioMessage(999, profile='h1')
                message.temperature(*apply_func(func_name, x, 4))
                message.humidity(*apply_func(func_name, x, 2))
                message.weight(math_func(func_name, x))
                #pprint(message)

                if logger.level != logging.DEBUG:
                    sys.stderr.write('.')

                # publish to MQTT
                self.publish_real(message.encode())

                time.sleep(interval)

        elif payload.startswith('json:'):
            json_payload = payload.replace('json:', '')
            data = json.loads(json_payload)

            node_id = data.get('node', '999')

            message = {
                'meta': {
                    'protocol': 'mqttkit1',
                    'network': str(self.protocol_class.network_id),
                    'gateway': str(self.protocol_class.gateway_id),
                    'node': node_id,
                    },
                'data': data,
                }

            self.mqtt.publish_json(message)

        elif payload.startswith('value:'):
            json_payload = payload.replace('value:', '')
            data = json.loads(json_payload)

            first = data.items()[0]
            name = first[0]
            value = first[1]

            #print name, value

            message = {
                'meta': {
                    'protocol': 'mqttkit1',
                    'network': str(self.protocol_class.network_id),
                    'gateway': str(self.protocol_class.gateway_id),
                    'node': 999,
                    },
                'data': data,
                }

            self.mqtt.publish_value(message, name, value)

        # Decode JobeeMonitor line format
        # id=Danvou;lux=13940.88;bmpP=997.03;bmpT=20.32;topT=0;entryT=0;h=70.55;siT=19.38;rainLevel=2.24;RainFall=466;milli=332143870;
        elif payload.startswith('id=') and ';' in payload:
            data = jobee_decode(payload)
            nodeid = data['id']
            del data['id']

            # Build an appropriate message from Jobee data
            message = {
                'meta': {
                    'network': str(self.protocol_class.network_id),
                    'gateway': str(self.protocol_class.gateway_id),
                    'node': nodeid,
                    },
                'data': data
            }
            self.mqtt.publish_flexible(message)

        else:
            self.publish_real(payload)


        self.mqtt.mqttc.loop_stop()

    def publish_real(self, payload):

        # decode wire format
        data = self.protocol_class.decode(payload)
        logger.debug('data: {}'.format(data))

        if self.protocol_class.VERSION == 1:
            data = self.protocol_class.to_v2(data)

        # publish to MQTT
        self.mqtt.publish_flexible(data, bencode_raw=payload)

    def __del__(self):
        if hasattr(self, 'mqtt'):
            logger.info('Disconnecting from MQTT broker')
            self.mqtt.close()


def random_temp():
    return round(random.uniform(7, 26), 2)

def random_hum():
    return round(random.uniform(3, 11), 2)

def random_weight():
    return round(random.uniform(100, 130), 2)

def scale_100(temp):
    return int(temp * 100)


if __name__ == '__main__':
    DataToMQTT(mqtt_broker=sys.argv[1], protocol=1).setup().publish(sys.argv[2])



import os
from setuptools import setup, find_packages

here = os.path.abspath(os.path.dirname(__file__))
README = open(os.path.join(here, 'README.rst')).read()
#CHANGES = open(os.path.join(here, 'CHANGES.rst')).read()

requires = [
    'pyserial==2.7',
    'paho-mqtt==1.3.1',
    'bencode==1.0',
    'docopt==0.6.2',
    'appdirs>=1.4.0',
    'json-store==2.1',
    # 'terkin==0.0.1',
]

test_requires = [
    'nose==1.3.7',
    'coverage==4.0.1',
]

setup(name='beradio',
    version='0.12.0',
    description='BERadio',
    long_description=README,
    license="AGPL 3, EUPL 1.2",
    classifiers=[
        "Development Status :: 4 - Beta",
        "Environment :: Console",
        "Intended Audience :: Developers",
        "Intended Audience :: Information Technology",
        "Intended Audience :: Science/Research",
        "License :: OSI Approved :: European Union Public Licence 1.2 (EUPL 1.2)",
        "License :: OSI Approved :: GNU Affero General Public License v3 or later (AGPLv3+)",
        "Natural Language :: English",
        "Operating System :: MacOS :: MacOS X",
        "Operating System :: POSIX :: Linux",
        "Programming Language :: Python",
        "Topic :: Communications",
        "Topic :: Internet",
        "Topic :: Scientific/Engineering",
        "Topic :: Software Development :: Libraries",
        "Topic :: System :: Archiving",
        "Topic :: Terminals :: Serial",
        "Topic :: Text Processing",
        "Topic :: Utilities",
    ],
    author='The Hiveeyes Developers',
    author_email='hello@hiveeyes.org',
    url='https://hiveeyes.org/docs/beradio/',
    keywords='protocol rfm serial mqtt Bencode BERadio',
    packages=find_packages(),
    include_package_data=True,
    zip_safe=False,
    test_suite='beradio',
    install_requires=requires,
    tests_require=test_requires,
    extras_require={
        'test': test_requires,
    },
    entry_points={
        'console_scripts': [
            'beradio            = beradio.commands:beradio_cmd',
            'bdecode            = beradio.commands:bdecode_cmd',
            'bencode            = beradio.commands:bencode_cmd',
            'bemqtt             = beradio.commands:bemqtt_cmd',
        ],
    },
)



# -*- coding: utf-8 -*-
# (c) 2015 Richard Pobering <richard@hiveeyes.org>
# (c) 2015-2018 Andreas Motl <andreas@hiveeyes.org>
import os
import json
import time
import logging
import threading
from copy import deepcopy
from urlparse import urlsplit
from datetime import datetime
from collections import OrderedDict

import paho.mqtt.client as mqtt

from beradio import program_name
from beradio.network import protocol_factory
from beradio.util import get_hostname

logger = logging.getLogger(__name__)


class MQTTAdapter(object):

    # The value of rc indicates success or not:
    RC_STATUS_MAP = {
        0: 'Connection successful',
        1: 'Connection refused - incorrect protocol version',
        2: 'Connection refused - invalid client identifier',
        3: 'Connection refused - server unavailable',
        4: 'Connection refused - bad username or password',
        5: 'Connection refused - not authorised',
        # 6 - 255: Currently unused.
    }

    def __init__(self, uri, keepalive=60, topic=None, enable_heartbeat=False):

        # Decode MQTT connection URL, e.g.
        # mqtt://test@example.org:12345@mqtt.example.org/testdrive/
        address = urlsplit(uri)

        # Propagate components of connection string
        self.host = address.hostname
        self.port = address.port or 1883
        self.username = address.username
        self.password = address.password
        self.keepalive = keepalive

        # Derive MQTT topic from connection URL
        # TODO: Revisit and check whether the "topic" parameter still makes sense at all?
        self.topic = topic or address.path.lstrip('/').rstrip('/')

        # Whether to send out high-level ping probes
        self.enable_heartbeat = enable_heartbeat

        # Compute MQTT client id
        program = program_name()
        hostname = get_hostname()
        pid = os.getpid()
        self.client_id = '{program}:{hostname}:{pid}'.format(program=program, hostname=hostname, pid=pid)

        # Attempt to connect to MQTT broker
        self.connect()

    def connect(self):

        # Define userdata which will be attached to each message
        userdata = {'gateway': True}

        # Create a MQTT client object
        self.mqttc = mqtt.Client(client_id=self.client_id, clean_session=True, userdata=userdata)

        # Optionally add authentication into the mix
        if self.username:
            self.mqttc.username_pw_set(self.username, self.password)

        # Connect to broker
        logger.info('Connecting to MQTT broker with '
                    'host={}, port={}, keepalive={}'.format(self.host, self.port, self.keepalive))
        self.mqttc.connect(self.host, self.port, self.keepalive)

        # Attach MQTT callbacks
        self.mqttc.on_connect = self.on_connect
        self.mqttc.on_disconnect = self.on_disconnect
        self.mqttc.on_publish = self.on_publish
        self.mqttc.on_subscribe = self.on_subscribe
        self.mqttc.on_unsubscribe = self.on_unsubscribe
        self.mqttc.on_message = self.on_message

    def get_status(self):
        status = OrderedDict()
        status['host'] = self.host
        status['port'] = self.port
        status['username'] = self.username
        return status

    def close(self):
        logger.info('Closing connection to MQTT broker')
        if self.enable_heartbeat:
            self.set_offline()
            time.sleep(0.25)
        self.mqttc.disconnect()

    def publish(self, topic, data, retain=False):
        logger.debug('Publishing to topic {}: data={}'.format(topic, data))
        return self.mqttc.publish(topic, data, retain=retain)

    def subscribe(self, topic):
        logger.info('Subscribing to topic {}'.format(topic))
        return self.mqttc.subscribe(topic)

    # MQTT callbacks
    def on_connect(self, client, userdata, flags, rc):

        # Get status information
        status = self.get_status()
        status['userdata'] = userdata
        status['flags'] = flags
        status['rc'] = rc
        status['rc_message'] = self.RC_STATUS_MAP.get(rc)

        if rc == 0:
            status['status'] = 'ok'
            logger.info('Connection to MQTT broker succeeded: {}'.format(json.dumps(status)))

            # Publish ping/alive message
            if self.enable_heartbeat:
                self.publish_ping()

        else:
            status['status'] = 'error'
            logger.error('Connection to MQTT broker failed: {}'.format(json.dumps(status)))

    def on_disconnect(self, client, userdata, *args):
        logger.info('Disconnection from MQTT broker succeeded')

    def on_publish(self, client, userdata, mid):
        #logger.info("Message " + str(mid) + " published.")
        pass

    def on_subscribe(self, client, userdata, mid, granted_qos):
        logger.info("Subscribe with mid " + str(mid) + " received")

    def on_unsubscribe(self, client, userdata, mid):
        logger.info("Unsubscribe with mid " + str(mid) + " received")

    def on_message(self, client, userdata, message):
        logger.info(u'Message received on topic {} with QoS {} and payload {}'.format(
            message.topic, str(message.qos), message.payload))

    def publish_ping(self):
        raise NotImplementedError()

    def set_offline(self):
        raise NotImplementedError()


class MQTTPublisher(object):

    topic_template = None

    def __init__(self, mqtt_adapter, realm, message, retain=False):
        self.mqtt = mqtt_adapter
        self.realm = realm or 'default'
        self.message = message
        self.retain = retain

    def publish(self, name, value):
        topic = self.compute_topic(name=name, metadata=self.message['meta'])
        self.mqtt.publish(topic, value, retain=self.retain)

    def compute_topic(self, name, metadata):
        tplvars = {}
        tplvars.update({'name': name})
        tplvars.update({'realm': self.realm})
        tplvars.update(metadata)
        topic = self.topic_template.format(**tplvars)
        #logger.info('topic: {}'.format(topic))
        return topic

    def scalar(self, name, value):
        self.publish(name, value)

    def field(self, fieldname):
        try:
            value = self.message['data'][fieldname]
        except KeyError:
            logger.warning(u'Could not find field "{}" to publish'.format(fieldname))
            return

        self.publish(fieldname, value)

    def all_fields(self):
        for key in self.message['data'].keys():
            self.field(key)

    def json(self, name, data):
        payload = json.dumps(data)
        self.publish(name, payload)

    def set_will(self, name, data):
        payload = json.dumps(data)
        topic = self.compute_topic(name=name, metadata=self.message['meta'])
        self.mqtt.mqttc.will_set(topic, payload=payload)


class BERadioMQTTPublisher(MQTTPublisher):
    topic_template = u'{realm}/{network}/{gateway}/{node}/{name}'


class BERadioMQTTAdapter(MQTTAdapter):

    STATUS_INTERVAL = 5 * 60
    STATUS_SUFFIX = 'status.json'

    def __init__(self, *args, **kwargs):
        kwargs.setdefault('enable_heartbeat', False)
        MQTTAdapter.__init__(self, *args, **kwargs)
        if self.enable_heartbeat:
            self.pinger = BERadioPinger(self, interval=self.STATUS_INTERVAL)

    def publish_flexible(self, message, do_json=True, bencode_raw=None):

        publisher = BERadioMQTTPublisher(self, self.topic, message)

        # Publish data in JSON format
        if do_json:

            message_block = deepcopy(message)

            # 2015-11-14: Add nanosecond timestamp to JSON message to improve acquisition precision
            if 'time' in message_block['meta']:
                message_block['data']['time'] = message_block['meta']['time']

            publisher.json('data.json', message_block['data'])

        # Publish data in Bencode format
        if bencode_raw:
            publisher.scalar('data.beradio', bencode_raw)

        # Publish all data values to discrete MQTT topics
        # TODO: Conditionally reenable again
        #publisher.all_fields()

    def publish_json(self, message):
        publisher = BERadioMQTTPublisher(self, self.topic, message)
        publisher.json('data.json', message['data'])

    def publish_value(self, message, name, value):
        publisher = BERadioMQTTPublisher(self, self.topic, message)
        publisher.scalar('data/{}'.format(name), value)

    def publish_ping(self):

        logger.info('Publishing ping message')

        # Get "ping" payload
        info = self.get_status_data('online')

        # Publish to "ping.json" in the context of the designated channel
        message = protocol_factory().get_envelope(node='gateway')
        publisher = BERadioMQTTPublisher(self, self.topic, message, retain=True)
        publisher.json(self.STATUS_SUFFIX, info)

        #self.set_testament()

    def set_offline(self):

        # Get "ping" payload
        info = self.get_status_data('offline')

        # Define testament to be published to "ping.json" in the context of the designated channel
        message = protocol_factory().get_envelope(node='gateway')
        publisher = BERadioMQTTPublisher(self, self.topic, message, retain=True)
        publisher.json(self.STATUS_SUFFIX, info)

    def set_testament(self):

        print 'set_testament'

        # Get "ping" payload
        info = self.get_status_data('offline')

        # Define testament to be published to "ping.json" in the context of the designated channel
        message = protocol_factory().get_envelope(node='gateway')
        publisher = BERadioMQTTPublisher(self, self.topic, message, retain=True)
        publisher.set_will(self.STATUS_SUFFIX, info)

    def get_status_data(self, status):
        """
        Define "ping" payload
        """
        info = OrderedDict()
        info['status'] = status
        info['program'] = program_name(with_version=True)
        info['date'] = datetime.utcnow().isoformat()
        return info

    def subscribe(self, subtopic=None):
        components = []
        if self.topic:
            components.append(self.topic)
        if subtopic:
            components.append(subtopic)
        topic = '/'.join(components)
        return MQTTAdapter.subscribe(self, topic)


class BERadioPinger(object):

    def __init__(self, mqtt_adapter, interval=30):
        self.mqtt_adapter = mqtt_adapter
        self.interval = interval
        self.restart()

    def restart(self):
        self.timer = threading.Timer(self.interval, self.ping)
        self.timer.setDaemon(True)
        self.timer.start()

    def ping(self):
        self.mqtt_adapter.publish_ping()
        self.restart()
